@@ -1,6 +1,7 @@
 #include "SFSI.h"
 //#include "utils.c"
 
+
 //====================================================================
 // Recursive quantities for the GEMMA algorithm:
 // a'Pib, a'PiPib, and a'PiPiPib, for any a and b
@@ -113,15 +114,15 @@ double det_WtHinvW(int i, int n, double *w, double *dbar){
 //====================================================================
 // Log Likelihood (ML) and Restricted Log Likelihood (REML)
 //====================================================================
-double logLik(double ratio, int n, int p, double *Uty, double *UtX,
+double logLik(double ratio, int n, int p, int nPC, double *Uty, double *UtX,
               double *d, double pi, double *dbar)
 {
   int k;
   double logdetH = 0;   // log(det(H)) = log(prod(ratio*d+1))
 
-  for(k=0; k<n; k++){
-    dbar[k] = 1/(ratio*d[k] + 1);
-    logdetH += log(ratio*d[k] + 1);
+  for(k=0; k<nPC; k++){
+    dbar[k] = 1/(ratio*d[k] + 1);    // dbar = 1 for small d
+    logdetH += log(ratio*d[k] + 1);  // log(.) = 0 for small d
   }
 
   // Remove the factor 0.5: 0.5*n*log(0.5*n/pi) - 0.5*n - 0.5*logdetH - 0.5*n*log(ytPy)
@@ -133,15 +134,15 @@ double logLik(double ratio, int n, int p, double *Uty, double *UtX,
 // df = n-p-1 because they adjust single marker regression as well
 // thus substracting 1 df. Here we do not adjust SMR so df = n-p
 
-double logResLik(double ratio, int n, int p, double *Uty, double *UtX,
+double logResLik(double ratio, int n, int p, int nPC, double *Uty, double *UtX,
                  double *d, double pi, double *dbar)
 {
   int k;
   double logdetH = 0;   // log(det(H)) = log(prod(ratio*d+1))
 
-  for(k=0; k<n; k++){
-    dbar[k] = 1/(ratio*d[k] + 1);
-    logdetH += log(ratio*d[k] + 1);
+  for(k=0; k<nPC; k++){
+    dbar[k] = 1/(ratio*d[k] + 1);    // dbar = 1 for small d
+    logdetH += log(ratio*d[k] + 1);  // log(.) = 0 for small d
   }
 
   // Remove the factor 0.5 and the term +log(detXtX)
@@ -153,12 +154,12 @@ double logResLik(double ratio, int n, int p, double *Uty, double *UtX,
 //====================================================================
 // First derivative of the Log Likelihood (ML)
 //====================================================================
-double dlogLik(double ratio, int n, int p, double *Uty, double *UtX,
+double dlogLik(double ratio, int n, int p, int nPC, double *Uty, double *UtX,
                double *d, double pi, double *dbar)
 {
   int k;
-  for(k=0; k<n; k++){
-    dbar[k] = 1/(ratio*d[k] + 1);
+  for(k=0; k<nPC; k++){
+    dbar[k] = 1/(ratio*d[k] + 1);  // dbar = 1 for small d
   }
 
   double Tr_HinvG = (n - dsum(n,dbar))/ratio;  // sum(diag(Hinv%*%G)) = (n-sumd)/ratio
@@ -172,13 +173,13 @@ double dlogLik(double ratio, int n, int p, double *Uty, double *UtX,
 //====================================================================
 // First derivative of the Log-restricted Likelihood (REML)
 //====================================================================
-double dlogResLik(double ratio, int n, int p, double *Uty, double *UtX,
+double dlogResLik(double ratio, int n, int p, int nPC, double *Uty, double *UtX,
                   double *d, double pi, double *dbar)
 {
   int k;
 
-  for(k=0; k<n; k++){
-    dbar[k] = 1/(ratio*d[k] + 1);
+  for(k=0; k<nPC; k++){
+    dbar[k] = 1/(ratio*d[k] + 1);  // dbar = 1 for small d
   }
 
   double Tr_PG = (n-p-tr_P(p-1,n,UtX,dbar))/ratio;  // sum(diag(Px%*%G))
@@ -192,20 +193,17 @@ double dlogResLik(double ratio, int n, int p, double *Uty, double *UtX,
 //====================================================================
 // TRUE if x1*x2 negative
 int RootBracketed(double x1,double x2) {
-  int result;
-
+  //int result;
   if((sign(x1)*sign(x2)) <= 0){
-    result = 1;
+    return(1);
   }else{
-    result = 0;
+    return(0);
   }
-
-  return result;
 }
 
 // Define fun and dfun that will be later either REML or ML
-double (*fun)(double, int, int, double*, double*, double*, double, double*) = NULL;
-double (*dfun)(double, int, int, double*, double*,double*, double, double*) = NULL;
+double (*fun)(double, int, int, int, double*, double*, double*, double, double*) = NULL;
+double (*dfun)(double, int, int, int, double*, double*,double*, double, double*) = NULL;
 
 /*******************************************************
 *              Brent Method Function                   *
@@ -225,14 +223,14 @@ double (*dfun)(double, int, int, double*, double*,double*, double, double*) = NU
 * OUTPUTS: The function returns the root value         *
 *          ValueAtRoot : value of f(root)              *
 *          niter    : number of done iterations        *
-*          error    : =0, all OK                       *
+*          info     : =0, all OK                       *
 *                   : =1, no root found in interval    *
 *                   : =2, no more iterations !         *
 *******************************************************/
 double BrentRoots(double x1, double x2,
                   double Tolerance, int maxIterations,
-                  double *valueAtRoot, int *niter, int *error,
-                  int n, int p, double *Uty, double *UtX,
+                  double *valueAtRoot, int *niter, int *info,
+                  int n, int p, int nPC, double *Uty, double *UtX,
                   double *d, double pi, double *dbar){
 
   double FPP = DBL_EPSILON; // 1e-11;
@@ -242,12 +240,17 @@ double BrentRoots(double x1, double x2,
 
   i = 0;
   done = 0;
-  *error = 0;
+  info[0] = 0;
   result = NA_REAL;
 
-  AA = x1;  BB = x2;
-  FA = dfun(AA, n, p, Uty, UtX, d, pi, dbar);
-  FB = dfun(BB, n, p, Uty, UtX, d, pi, dbar);
+  AA = x1;
+  BB = x2;
+  FA = dfun(AA, n, p, nPC, Uty, UtX, d, pi, dbar);
+  FB = dfun(BB, n, p, nPC, Uty, UtX, d, pi, dbar);
+
+  // Initialize these as lines below might not be
+  CC = AA; FC = FA; DD = BB - AA; EE = DD;
+
   if(RootBracketed(FA,FB)){
     FC = FB;
     do{
@@ -263,7 +266,7 @@ double BrentRoots(double x1, double x2,
       if((fabs(xm) <= Tol1) || (fabs(FA) < nearzero)){
         result = BB;
         done = 1;
-        *valueAtRoot = dfun(result, n, p, Uty, UtX, d, pi, dbar);
+        valueAtRoot[0] = dfun(result, n, p, nPC, Uty, UtX, d, pi, dbar);
         //Rprintf("A root has been found\n");
       }else{
         if((fabs(EE) >= Tol1) && (fabs(FA) > fabs(FB))){
@@ -299,105 +302,67 @@ double BrentRoots(double x1, double x2,
              BB = BB - fabs(Tol1);
           }
         }
-        FB = dfun(BB, n, p, Uty, UtX, d, pi, dbar);
+        FB = dfun(BB, n, p, nPC, Uty, UtX, d, pi, dbar);
         i++;
       }
 	  }
     while((!done) && (i < maxIterations));
-    if(i >= maxIterations) *error = 2;
+    if(i >= maxIterations) info[0] = 2;
   }else{
     //Rprintf("Values f(x) at bounds (%f,%f) are NOT of opposite signs\n",x1,x2);
-    *error = 1;
+    info[0] = 1;
   }
-  *niter = i;
+  niter[0] = i;
   return result;
 }
 
-SEXP R_solve_mixed(SEXP N_, SEXP ratio_, SEXP trn_, SEXP ytrn_,
-            SEXP X_, SEXP Z_, SEXP K_, SEXP U_,
-            SEXP d_, SEXP bounds_, SEXP tol_,
-            SEXP maxiter_, SEXP dmin_, SEXP isREML_,
-            SEXP isEigen_, SEXP BLUE_, SEXP BLUP_)
+SEXP R_solve_mixed(SEXP N_, SEXP n_, SEXP nPC_, SEXP ratio_,
+                   SEXP Uty_, SEXP UtX_,
+                   SEXP d_, SEXP bounds_, SEXP tol_,
+                   SEXP maxiter_, SEXP isREML_, SEXP BLUE_)
 {
-    int j;
-    int *trn;
-    double x1, x2, value, ratio, varE, varU, h2;
-    double *ytrn, *X, *d, *U, *work, *dbar, *tmp1, *tmp2, *tmp3;
-    double *Uty, *UtX, *yHat;
-    double *bounds;
+    int j, intvalue;
     double one = 1;
-    int inc1 = 1;
-    int intval;
-    SEXP list, uHat_, yHat_, bHat_;
-    int nprotect = 8;
+    int nprotect = 4;
 
-    double tol=NUMERIC_VALUE(tol_);
-    double dmin=NUMERIC_VALUE(dmin_);
-    int maxiter=INTEGER_VALUE(maxiter_);
-    int N=INTEGER_VALUE(N_);
-    int ntrn=Rf_length(trn_);
-    int p=Rf_ncols(X_);
-    int isREML=asLogical(isREML_);
-    int isEigen=asLogical(isEigen_);
-    int BLUE=asLogical(BLUE_);
-    int BLUP=asLogical(BLUP_);
-    int nintervals=Rf_length(bounds_)-1;
+    double tol = NUMERIC_VALUE(tol_);
+    int maxiter = INTEGER_VALUE(maxiter_);
+    int N = INTEGER_VALUE(N_);
+    int n = INTEGER_VALUE(n_);
+    int nPC = INTEGER_VALUE(nPC_);
+    int p = Rf_ncols(UtX_);
+    int isREML = asLogical(isREML_);
+    int BLUE = asLogical(BLUE_);
+    int nintervals = Rf_length(bounds_)-1;
 
-    PROTECT(trn_=AS_INTEGER(trn_));
-    trn=INTEGER_POINTER(trn_);
+    PROTECT(Uty_ = AS_NUMERIC(Uty_));
+    double *Uty = NUMERIC_POINTER(Uty_);
 
-    PROTECT(ytrn_=AS_NUMERIC(ytrn_));
-    ytrn=NUMERIC_POINTER(ytrn_);
+    PROTECT(UtX_ = AS_NUMERIC(UtX_));
+    double *UtX = NUMERIC_POINTER(UtX_);
 
-    PROTECT(X_=AS_NUMERIC(X_));
-    X=NUMERIC_POINTER(X_);
+    PROTECT(d_ = AS_NUMERIC(d_));
+    double *d = NUMERIC_POINTER(d_);
 
-    PROTECT(d_=AS_NUMERIC(d_));
-    d=NUMERIC_POINTER(d_);
+    PROTECT(bounds_ = AS_NUMERIC(bounds_));
+    double *bounds = NUMERIC_POINTER(bounds_);
 
-    PROTECT(U_=AS_NUMERIC(U_));
-    U=NUMERIC_POINTER(U_);
-
-    PROTECT(bounds_=AS_NUMERIC(bounds_));
-    bounds=NUMERIC_POINTER(bounds_);
-
-    double dfx0 = NA_REAL;
     int convergence = NA_INTEGER;
     int status = NA_INTEGER;
-    int ndsmall = NA_INTEGER;
-    int niter = NA_INTEGER;
-    int error = NA_INTEGER;
     double pi = M_PI, eps = DBL_EPSILON;
 
-    Uty = (double *) R_alloc(ntrn, sizeof(double));
-    UtX = (double *) R_alloc(ntrn*p, sizeof(double));
+    intvalue = N>(p*p) ? N : p*p;
+    double *dbar = (double *) R_alloc(n, sizeof(double));
+    double *work = (double *) R_alloc(intvalue, sizeof(double));
+    double *tmp1 = (double *) R_alloc(intvalue, sizeof(double));
+    double *tmp2 = (double *) R_alloc(intvalue, sizeof(double));
+    double *tmp3 = (double *) R_alloc(intvalue, sizeof(double));
 
-    yHat_ = PROTECT(Rf_allocVector(REALSXP, N));
-    yHat = NUMERIC_POINTER(yHat_);
-
-    // Check for small eigenvalues
-    ndsmall=0;
-    for(j=0; j<ntrn; j++){
-      if(d[j] < dmin){
-        ndsmall++;
-        d[j] = 0;
-      }
+    for(j=0; j<n; j++){
+      dbar[j] = 1;
     }
 
-    intval = N>(p*p) ? N : p*p;
-    dbar=(double *) R_alloc(ntrn, sizeof(double));
-    work=(double *) R_alloc(intval, sizeof(double));
-    tmp1=(double *) R_alloc(intval, sizeof(double));
-    tmp2=(double *) R_alloc(intval, sizeof(double));
-    tmp3=(double *) R_alloc(intval, sizeof(double));
-
-    //Rprintf(" Calculating UtX and Uty...\n");
-    for(j=0; j<p; j++){
-      slice_matrix(N,X,tmp1,ntrn,trn,j,2); // X[trn,j]
-      matrix_vector_product(ntrn,ntrn,&one,U,tmp1,1,UtX+ntrn*j,1); // UtX[,j]=U'X[trn,j]
-    }
-    matrix_vector_product(ntrn,ntrn,&one,U,ytrn,1,Uty,1); // U'ytrn
-
+    double ratio = NA_REAL;
     if(Rf_isNull(ratio_)){ // Perform likelihood maximization
       // Define function for REML or ML
       if(isREML){
@@ -408,40 +373,40 @@ SEXP R_solve_mixed(SEXP N_, SEXP ratio_, SEXP trn_, SEXP ytrn_,
         dfun = &dlogLik;
       }
 
-      double fx1, fx2, fx0, slope, dfx1, dfx2;
-      error = 1;
-      status = 0;
+      double x1, x2, fx1, fx2, slope, dfx1, dfx2, value;
+      int niter;
       convergence = 0;
 
       // Evaluate the likelihood at the bounds and get slope
       x1 = bounds[0];
       x2 = bounds[nintervals];
-      fx1 = fun(x1, ntrn, p, Uty, UtX, d, pi, dbar);
-      fx2 = fun(x2, ntrn, p, Uty, UtX, d, pi, dbar);
+      fx1 = fun(x1, n, p, nPC, Uty, UtX, d, pi, dbar);
+      fx2 = fun(x2, n, p, nPC, Uty, UtX, d, pi, dbar);
       slope = (fx2-fx1)/(x2-x1);
 
       //Rprintf(" Starting searching of optimal value ...\n");
       if(fabs(slope) < eps){
         ratio = x1;
-        convergence = 0;
         status = 1;
       }else{
         //Rprintf("Starting Brent's algorithm (within interval)...\n");
+        status = 0;
+        intvalue = 1;
         for(j=0; j<nintervals; j++){
-          dfx1 = dfun(bounds[j], ntrn, p, Uty, UtX, d, pi, dbar);
-          dfx2 = dfun(bounds[j+1], ntrn, p, Uty, UtX, d, pi, dbar);
+          dfx1 = dfun(bounds[j], n, p, nPC, Uty, UtX, d, pi, dbar);
+          dfx2 = dfun(bounds[j+1], n, p, nPC, Uty, UtX, d, pi, dbar);
 
           if(sign(dfx1)*sign(dfx2) <= 0){
-            value = BrentRoots(bounds[j], bounds[j+1], tol, maxiter,
-                             &dfx0, &niter, &error,
-                             ntrn, p, Uty, UtX, d, pi, dbar);
-            if(error != 1){
+            ratio = BrentRoots(bounds[j], bounds[j+1], tol, maxiter,
+                               &value, &niter, &intvalue,
+                               n, p, nPC, Uty, UtX, d, pi, dbar);
+            if(intvalue != 1){ // a root is found if intvalue = 0 or 2
               break;
             }
           }
         }
 
-        if(error == 1){
+        if(intvalue == 1){
           //Rprintf("No solution was found in the interval\n");
           //Rprintf("The coordinate bound yielding the larger likelihood is returned\n");
           if(fx1 > fx2){
@@ -453,10 +418,9 @@ SEXP R_solve_mixed(SEXP N_, SEXP ratio_, SEXP trn_, SEXP ytrn_,
           }
           convergence = 0;
         }else{
-          ratio = value;
-          fx0 = fun(ratio, ntrn, p, Uty, UtX, d, pi, dbar);
-          if(fx0 < fmax2(fx1, fx2)){  // Degenerated solution
-            //Rprintf("A degenerated solution was found: x=%f and f(x)=%f\n",ratio,fx0);
+          value = fun(ratio, n, p, nPC, Uty, UtX, d, pi, dbar); // likelihood at root
+          if(value < fmax2(fx1, fx2)){  // Degenerated solution
+            //Rprintf("A degenerated solution was found: x=%f and f(x)=%f\n",ratio,value);
             if(fx1 > fx2){
               ratio = x1;
               status = 3;
@@ -466,8 +430,8 @@ SEXP R_solve_mixed(SEXP N_, SEXP ratio_, SEXP trn_, SEXP ytrn_,
             }
             convergence = 0;
           }else{
-            //Rprintf("A good solution was found: x=%f and f(x)=%f\n",ratio,fx0);
-            convergence = (error ==  0 ? 1 : 0);
+            //Rprintf("A good solution was found: x=%f and f(x)=%f\n",ratio,value);
+            convergence = (intvalue ==  0 ? 1 : 0);
             if(!convergence){
               status = 2;
             }
@@ -480,119 +444,62 @@ SEXP R_solve_mixed(SEXP N_, SEXP ratio_, SEXP trn_, SEXP ytrn_,
     }
 
     // Variance components and fixed effects
-    for(j=0; j<ntrn; j++){
+    for(j=0; j<nPC; j++){
       dbar[j] = 1/(ratio*d[j] + 1);
-      tmp1[j] = Uty[j]*dbar[j];
     }
 
+    SEXP bHat_;
     if(BLUE){
-      double *bHat;
-
+      //Rprintf(" Calculating fixed effects b ...\n");
       bHat_ = PROTECT(Rf_allocVector(REALSXP, p));
-      bHat = NUMERIC_POINTER(bHat_);
+      double *bHat = NUMERIC_POINTER(bHat_);
       nprotect++;
 
-      crossproduct(ntrn,1,p,tmp1,UtX,tmp2);   // tmp2=t(Uty*dbar)%*%UtX
-      crossproduct_scale(ntrn,p,p,UtX,dbar,UtX,tmp1,work); // tmp1 = UtX' dbar UtX
-      invert_matrix(p,tmp1,tmp3,&eps,work);  // tmp3=solve(UtX'*dbar*UtX)
-      matrix_vector_product(p,p,&one,tmp3,tmp2,1,bHat,0); // b = solve(UtX'*dbar*UtX)*tmp2
-      matrix_vector_product(N,p,&one,X,bHat,1,yHat,0);    // yHat = Xb
-
-      for(j=0; j<ntrn; j++){  // ytrn* = ytrn - X[trn,]b
-        ytrn[j]-=yHat[trn[j]];
+      // b = solve(UtX' dbar UtX) t(Uty*dbar) UtX
+      for(j=0; j<n; j++){
+        tmp1[j] = Uty[j]*dbar[j];
       }
+      crossproduct(n,1,p,tmp1,UtX,tmp2);   // tmp2 = t(tmp1)%*%UtX
+      crossproduct_scale(n,p,p,UtX,dbar,UtX,tmp1,work); // tmp1 = UtX' dbar UtX
+
+      intvalue = 0;
+      invert_matrix(p,tmp1,tmp3,&eps,work,&intvalue);     // tmp3 = solve(tmp1)
+      if(intvalue != 0){
+        UNPROTECT(nprotect);
+        return(ScalarInteger(5));  // Status 5: Matrix X is not full rank
+      }
+
+      matrix_vector_product(p,p,&one,tmp3,tmp2,1,bHat,0); // b = tmp3*tmp2
     }else{
       bHat_ = R_NilValue;
-      memset(yHat, 0, N*sizeof(double));
     }
 
-    varE = atPb(p-1,ntrn,Uty,Uty,UtX,dbar)/(isREML ? ntrn-p : ntrn);
-    varU = ratio*varE;
-    h2 = varU/(varU + varE);
+    double varE = atPb(p-1,n,Uty,Uty,UtX,dbar)/(isREML ? n-p : n);
+    double varU = ratio*varE;
+    double h2 = varU/(varU + varE);
 
-    if(BLUP){
-      double *Z, *K, *M, *uHat;
-      int nu = Rf_isNull(Z_) ? N : Rf_ncols(Z_);
+    SEXP list_ = PROTECT(Rf_allocVector(VECSXP, 7));
+    SET_VECTOR_ELT(list_, 0, ScalarReal(ratio));
+    SET_VECTOR_ELT(list_, 1, ScalarInteger(convergence));
+    SET_VECTOR_ELT(list_, 2, bHat_);
+    SET_VECTOR_ELT(list_, 3, ScalarReal(varU));
+    SET_VECTOR_ELT(list_, 4, ScalarReal(varE));
+    SET_VECTOR_ELT(list_, 5, ScalarReal(h2));
+    SET_VECTOR_ELT(list_, 6, ScalarInteger(status));
 
-      uHat_ = PROTECT(Rf_allocVector(REALSXP, nu));
-      uHat = NUMERIC_POINTER(uHat_);
-      nprotect++;
-
-      M = (double *) R_alloc(N*(ntrn>nu ? ntrn : nu), sizeof(double));
-
-      if(!Rf_isNull(Z_)){
-        PROTECT(Z_=AS_NUMERIC(Z_));
-        Z = NUMERIC_POINTER(Z_);
-        nprotect++;
-      }
-      if(!Rf_isNull(K_)){
-        PROTECT(K_=AS_NUMERIC(K_));
-        K = NUMERIC_POINTER(K_);
-        nprotect++;
-      }
-
-      memset(M, 0, ntrn*ntrn*sizeof(double));
-      if(isEigen){
-        // Matrix B = KZ'Hinv = U diag{d*ratio/(d*ratio+1)} U'
-        for(j=0; j<ntrn; j++){
-          tmp1[j] = d[j]*ratio/(ratio*d[j] + 1);
-        }
-        tcrossproduct_scale(ntrn,ntrn,ntrn,U,tmp1,U,M,work);
-        matrix_vector_product(ntrn,ntrn,&one,M,ytrn,1,uHat,0);
-      }else{
-        //  Hinv = U diag{1/(theta+d)} U' = U diag{ratio/(ratio*d+1)} = UDU'
-        for(j=0; j<ntrn; j++){
-          tmp1[j] = ratio/(ratio*d[j] + 1);
-        }
-
-        tcrossproduct_scale(ntrn,ntrn,ntrn,U,tmp1,U,M,work); // Hinv = UDU'
-        matrix_vector_product(ntrn,ntrn,&one,M,ytrn,1,tmp1,0); // tmp1=Hinv%*%(y-Xb)
-
-        if(Rf_isNull(Z_) && Rf_isNull(K_)){
-          memset(uHat, 0, nu*sizeof(double));
-          for(j=0; j<ntrn; j++){
-            uHat[trn[j]] = tmp1[j];
-          }
-        }else{
-          if(Rf_isNull(Z_)){   // Z=NULL, K=K:   u = K[,trn]*Hinv*(y-Xb)
-            matrix_vector_product_subset(N,N,K,tmp1,uHat,0,NULL,ntrn,trn,0,work);
-          }else{
-            if(Rf_isNull(K_)){  // Z=Z, K=NULL:   u = Z[trn,]'Hinv*(y-Xb)
-              matrix_vector_product_subset(N,nu,Z,tmp1,uHat,ntrn,trn,0,NULL,1,work);
-
-            }else{  // u = KZ[trn,]'Hinv*(y-Xb)
-              matrix_vector_product_subset(N,nu,Z,tmp1,tmp2,ntrn,trn,0,NULL,1,work);
-              matrix_vector_product(nu,nu,&one,K,tmp2,1,uHat,0);
-            }
-          }
-        }
-      }
-
-      if(Rf_isNull(Z_)){
-        F77_NAME(daxpy)(&N,&one,uHat,&inc1,yHat,&inc1);   // yHat = Xb + u
-      }else{
-        matrix_vector_product(N,nu,&one,Z,uHat,1,tmp1,0); // Zu
-        F77_NAME(daxpy)(&N,&one,tmp1,&inc1,yHat,&inc1);   // yHat = Xb + Zu
-      }
-    }else{
-      uHat_ = R_NilValue;
-    }
-
-    PROTECT(list = Rf_allocVector(VECSXP, 12));
-    SET_VECTOR_ELT(list, 0, ScalarReal(ratio));
-    SET_VECTOR_ELT(list, 1, ScalarReal(dfx0));
-    SET_VECTOR_ELT(list, 2, ScalarInteger(niter));
-    SET_VECTOR_ELT(list, 3, ScalarInteger(status));
-    SET_VECTOR_ELT(list, 4, ScalarInteger(convergence));
-    SET_VECTOR_ELT(list, 5, ScalarInteger(ndsmall));
-    SET_VECTOR_ELT(list, 6, ScalarReal(varU));
-    SET_VECTOR_ELT(list, 7, ScalarReal(varE));
-    SET_VECTOR_ELT(list, 8, ScalarReal(h2));
-    SET_VECTOR_ELT(list, 9, bHat_);
-    SET_VECTOR_ELT(list, 10, yHat_);
-    SET_VECTOR_ELT(list, 11, uHat_);
+    // Set dimnames for outputs
+    SEXP names_ = PROTECT(Rf_allocVector(VECSXP, 7));
+    SET_VECTOR_ELT(names_, 0, mkChar("ratio"));
+    SET_VECTOR_ELT(names_, 1, mkChar("convergence"));
+    SET_VECTOR_ELT(names_, 2, mkChar("b"));
+    SET_VECTOR_ELT(names_, 3, mkChar("varU"));
+    SET_VECTOR_ELT(names_, 4, mkChar("varE"));
+    SET_VECTOR_ELT(names_, 5, mkChar("h2"));
+    SET_VECTOR_ELT(names_, 6, mkChar("status"));
+    Rf_setAttrib(list_, R_NamesSymbol, names_);
+    nprotect += 2;
 
     UNPROTECT(nprotect);
 
-    return(list);
+    return(list_);
 }
