@@ -46,7 +46,7 @@ SEXP R_updatebeta(SEXP XtX_, SEXP Xty_,
                   SEXP scale_, SEXP sd_, SEXP filename_,
                   SEXP doubleprecision_, SEXP verbose_)
 {
-    double L1, L2, maxerror, delta, bNew;
+    double L1, L2, maxerror, delta, bOLS, bNew;
     long long j;
     int k, iter;
     int varsize, vartype;
@@ -80,10 +80,10 @@ SEXP R_updatebeta(SEXP XtX_, SEXP Xty_,
     double *Xty = NUMERIC_POINTER(Xty_);
 
     int *df = (int *) R_alloc(nlambda, sizeof(int));
-    int *niter = (int *) R_alloc(nlambda, sizeof(int));      // Niter at each lambda
+    int *niter = (int *) R_alloc(nlambda, sizeof(int));     // Niter at each lambda
     double *error = (double *) R_alloc(nlambda, sizeof(double)); // Max error b0-bnew at each lambda
-    double *b = (double *) R_alloc(p, sizeof(double));   // Current b[j] values
-    double *bout = (double *) R_alloc(p, sizeof(double));  // Output
+    double *b = (double *) R_alloc(p, sizeof(double));      // Current b[j] values
+    double *bout = (double *) R_alloc(p, sizeof(double));   // Output
     double *XtyHatNoj = (double *) R_alloc(p, sizeof(double)); // XtyHatNoj[j] = {XtX[,j]'b - XtX[j,j]b[j]}
 
     for(k=0; k<nlambda; k++){
@@ -92,18 +92,14 @@ SEXP R_updatebeta(SEXP XtX_, SEXP Xty_,
 
     if(Rf_isNull(b0_)){
       memset(b, 0, sizeof(double)*p);           // Initialize all b[j] to zero
-      memset(XtyHatNoj, 0, sizeof(double)*p);  // Since all b[j] are initially 0, all XtyHatNoj[j] are so
+      memset(XtyHatNoj, 0, sizeof(double)*p);   // Since all b[j] are initially 0, all XtyHatNoj[j] are so
     }else{
       PROTECT(b0_ = AS_NUMERIC(b0_));
+      memcpy(b, NUMERIC_POINTER(b0_), sizeof(double)*p);
       nprotect++;
 
-      memcpy(b, NUMERIC_POINTER(b0_), sizeof(double)*p);
-      //delta = -1;
-      //double one = 1;
-      //matrix_vector_product(p,p,&one,XtX,b,1,XtyHatNoj,0);  // XtyHatNoj <- XtX b
-      //F77_NAME(daxpy)(&p, &delta, b, &inc1, XtyHatNoj, &inc1); // XtyHatNoj <- XtyHatNoj - b
-
-      for(j=0; j<p; j++){ // XtyHatNoj[j] = {XtX[,j]'b - XtX[j,j]b[j]}
+      // XtyHatNoj[j] = {XtX[,j]'b - XtX[j,j]b[j]}; here XtX[j,j]=1
+      for(j=0; j<p; j++){
         XtyHatNoj[j] = F77_NAME(ddot)(&p, XtX + p*j, &inc1, b, &inc1) - b[j];
       }
     }
@@ -112,85 +108,87 @@ SEXP R_updatebeta(SEXP XtX_, SEXP Xty_,
       //Rprintf(" Saving binary file info for B ...\n");
       varsize = doubleprecision ? sizeof(double) : sizeof(float);
       vartype = 3;
-      f=fopen(CHAR(STRING_ELT(filename_,0)),"wb");
+      f = fopen(CHAR(STRING_ELT(filename_,0)),"wb");
       fwrite(&p, sizeof(int), 1, f);
       fwrite(&nlambda, sizeof(int), 1, f);
       fwrite(&vartype, sizeof(int), 1, f);
       fwrite(&varsize, sizeof(int), 1, f);
     }else{
-      //Rprintf(" Allocating memory for B ...\n");  // Allocated memory is set to zero (as in calloc)
+      //Rprintf(" Allocating memory for B ...\n");
+      // Allocated memory is set to zero (as in calloc)
       B = (double *) R_Calloc(p*nlambda, double);
     }
 
     //Rprintf(" Starting beta updating ...\n");
     for(k=0; k<nlambda; k++)
     {
-        L1 = alpha*lambda[k];
-        L2 = (1-alpha)*lambda[k];
-        iter = 0;          // Set iter < maxiter to enter the WHILE
-        maxerror = tol + 1.0;   // Set a error > tol to enter the WHILE
-        while(iter<maxiter && maxerror>tol)
+      L1 = alpha*lambda[k];
+      L2 = (1-alpha)*lambda[k];
+      iter = 0;              // Set iter < maxiter to enter the WHILE
+      maxerror = tol + 1.0;  // Set a error > tol to enter the WHILE
+      while(iter<maxiter && maxerror>tol)
+      {
+        iter++;
+        maxerror = 0;
+        for(j=0; j<p; j++)
         {
-            iter++;
-            maxerror = 0;
-            for(j=0; j<p; j++)
-            {
-                // varj = XtX[p*j + j];  // Variance of predictor j
-                // bOLS = (Xty[j] - XtyHat[j])/varj;
-                bNew = soft_threshold(Xty[j] - XtyHatNoj[j], L1)/(1+L2);
+          // varj = XtX[p*j + j];  // Variance of predictor j
+          // bOLS = (Xty[j] - XtyHat[j])/varj;
+          bOLS = Xty[j] - XtyHatNoj[j];
+          bNew = soft_threshold(bOLS, L1)/(1+L2);
 
-                delta = bNew-b[j];
-                //Rprintf(" j=%d. XtyHatNoj=%1.8f bOLS=%1.8f  bNew=%1.8f  delta=%f\n",j+1,XtyHatNoj[j],Xty[j]-XtyHatNoj[j], bNew, delta);
-                if(fabs(delta)>0){ // Update only if there is a change
-                  // Update: XtyHatNoj[k] <- XtyHatNoj[k] + XtX[k,j]*(bNew[j]-b[j])
-                  F77_NAME(daxpy)(&p, &delta, XtX + p*j, &inc1, XtyHatNoj, &inc1);
+          delta = bNew-b[j];
+          //Rprintf(" j=%d. XtyHatNoj=%1.8f bOLS=%1.8f  bNew=%1.8f  delta=%f\n",j+1,XtyHatNoj[j], bOLS, bNew, delta);
+          if(fabs(delta)>0){ // Update only if there is a change
+            // Update: XtyHatNoj[k] <- XtyHatNoj[k] + XtX[k,j]*(bNew[j]-b[j])
+            F77_NAME(daxpy)(&p, &delta, XtX + p*j, &inc1, XtyHatNoj, &inc1);
 
-                  XtyHatNoj[j] -= delta; // Except for k=j. delta*varj
-                  if(fabs(delta)>maxerror){
-                    maxerror = fabs(delta);
-                  }
-                  b[j] = bNew;
-                }
+            XtyHatNoj[j] -= delta; // except for k=j. delta*varj
+            if(fabs(delta)>maxerror){
+              maxerror = fabs(delta);
             }
-        }
-        niter[k] = iter;
-        error[k] = maxerror;
-        F77_NAME(dcopy)(&p, b, &inc1, bout, &inc1);
-
-        if(scale){
-          for(j=0; j<p; j++){
-            bout[j] = bout[j]/sd[j];
+            b[j] = bNew;
           }
         }
+      }
+      niter[k] = iter;
+      error[k] = maxerror;
+      F77_NAME(dcopy)(&p, b, &inc1, bout, &inc1);
 
-        df[k] = 0;
+      if(scale){
         for(j=0; j<p; j++){
-          if(fabs(bout[j])>0) df[k]++;
+          bout[j] = bout[j]/sd[j];
         }
+      }
 
-        if(verbose){
-            Rprintf(" lambda[%d]=%1.8f  nsup=%5d  niter=%5d  Error=%G\n",k+1,lambda[k],df[k],iter,maxerror);
-            if(maxerror>tol){
-              Rprintf(" Warning: The process did not converge after %d iterations for lambda[%d]=%f\n",maxiter,k+1,lambda[k]);
-            }
-        }
+      df[k] = 0;
+      for(j=0; j<p; j++){
+        if(fabs(bout[j])>0) df[k]++;
+      }
 
-        if(save){
-          if(doubleprecision){
-            fwrite(bout, varsize, p, f);
-          }else{  // Cast to float one by one
-            for(j=0; j<p; j++){
-              valuefloat = bout[j];
-              fwrite(&valuefloat, varsize, 1, f);
-            }
+      if(verbose){
+          Rprintf(" lambda[%d]=%1.8f  nsup=%5d  niter=%5d  Error=%G\n",k+1,lambda[k],df[k],iter,maxerror);
+          if(maxerror>tol){
+            Rprintf(" Warning: The process did not converge after %d iterations for lambda[%d]=%f\n",maxiter,k+1,lambda[k]);
           }
-        }else{
-          F77_NAME(dcopy)(&p, bout, &inc1, B + p*k, &inc1);
-        }
+      }
 
-        if(dfmax<p && df[k]>=dfmax){
-          break;
+      if(save){
+        if(doubleprecision){
+          fwrite(bout, varsize, p, f);
+        }else{  // Cast to float one by one
+          for(j=0; j<p; j++){
+            valuefloat = bout[j];
+            fwrite(&valuefloat, varsize, 1, f);
+          }
         }
+      }else{
+        F77_NAME(dcopy)(&p, bout, &inc1, B + p*k, &inc1);
+      }
+
+      if(dfmax<p && df[k]>=dfmax){
+        break;
+      }
     }
 
     //Rprintf(" Writting results: lambda, nsup, B ...\n");
